/* Adaptive Population based Simplex
 * main.c
 * Copyright (C) 2013 Maurice <Maurice.Clerc@WriteMe.com>
	* 
 * APS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
	* 
 * APS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "main.h"

int main() 
{
 struct position bestBest;
 int d;
 double errorMeanBest[R_max]; 
 double errorMin;

 struct param param;
 struct problem pb;
 struct result R;
 int s;
 double success[funcMax];
 double successRate;
 double variance;

 double error;
 double errorMean;
 double evalMean;		// Mean number of evaluations
 int functionCode;
 int func[funcMax]; // List of functions (codes) to optimise
 int indFunc; 
 double meanSuccess;
 int nbFunc;
 int nFailure;		// Number of unsuccessful runs
 struct result result;
 int run;
 int runMax;
 int S;
 int t;
 double xMax,xMin;
 float z;
 double zz;

 E = exp ((long double) 1);  
 pi = acos ((long double) -1); 

 // Files
 f_run = fopen ("f_run.txt", "w");  
 f_synth = fopen ("f_synth.txt", "w");

 //-------------------------------------------------------------------Parameters
 param.randCase=10; // Select the RNG that will be used
 // 1 => KISS_PRNG
 //* 10 => Mersenne
 // -1 => standard C

 param.tryMore=1; 	// 0 => loop as soon as the current individual has been
 									//		improved, after Expansion or Contraction
 									// (pure standard APS)
 									//	1 => even if improvement after Expansion, try Contraction
 									//  2 => even if improvement after Contraction, try Local Search

 //------------------------------------------------------- Select the problem(s) 
 runMax = 10; // Numbers of runs
 if (runMax > R_max) runMax = R_max;

 // Functions to optimise
 nbFunc=1; //14. Number of functions
 func[0]=999; // 4  
 func[1]=18; // 11, -99
 func[2]=203;  // 15,
 func[3]=204;  // 17, 
 func[4]=205; // 18
 func[5]=206; // 20
 func[6]=21;  // 21
 func[7]=100; // 100
 func[8]=102; // 102 or 1020
 func[9]=103; // 103
 func[10]=104; // 104
 func[11]=105; // 105
 func[12]=106;	// 106	
 func[13]=28;	// 28	

 /* (see problemDef( ) for precise definitions)
 -100 Dummy (quick, and success rate=100)
 -99 Dummy (quick, and success rate = 0)
 -1  Constant. For test of biases
	0 Parabola (Sphere)
	1 Griewank
	 2 Rosenbrock (Banana)
	 3 Rastrigin
		4 Tripod (dimension 2)
		5 Ackley
		 6 Schwefel
		 7 Schwefel 1.2
		 8 Schwefel 2.2
		 9 Neumaier 3
		 10 G3
		 11 Network optimisation (Warning: see problemDef() and also perf() for
			problem elements (number of BTS and BSC)
			12 Schwefel
			 13 2D Goldstein-Price
			 14 Schaffer f6
			 15 Step	
			 16 Schwefel 2.21
			 17 Lennard-Jones
			 18 Gear train
			 19 Sine_sine function
			 20 Perm function
			 21 Compression Spring
			 22 Cellular phone (2D) 
			 23 Penalized
				24 Repulsion
				25 Pressure Vessel (penalty method)
				26 Ellipsoidal
				 27 Quadric
				 28 Frequency modulation sound parameter identification

				 CEC 2005 benchmark  (no more than 30D. See cec2005data.c)
				 100 F1 (shifted Parabola/Sphere) 
				 102 F6 (shifted Rosenbrock)  CEC 2005
					1020  "  but CEC 2008 (the offset is different)
					103 F9 (shifted Rastrigin) 
					104 F2 Schwefel 
					 105 F7 Griewank  (NOT rotated)
					 1050 "but CEC 2008 (the offset is different)
					 106 F8 Ackley  (NOT rotated)

					 CEC 2008 benchmark
						201 F1 Sphere
						202 F2 Schwefel
						203 F3 Rosenbrock
						204 F4 Rastrigin
						205 F5 Griewank
						206 F6 Ackley

						999 for tests
						*/ 
 //============================================================== Initialisation
 errMax=0; // Maximum error (for information)
 nbRand=0; // Number of random numbers used (for information)
 errMin=infinity;
 //============================================================ Loop on problems
 for(indFunc=0;indFunc<nbFunc;indFunc++) 
 {
	switch(param.randCase)  // Initialise the RNG
	{
	 case 1: // KISS PRNG
		randk_reset();
	 randk_seed();
	 break;

	 case 10: // Mersenne 64 bits
		init_genrand64(1294404794);
	 break;	;
	}

	switch(param.randCase) // "Warm up" the RNG 
	{
	 case 1:
		randk_warmup(10000);
		break;

	 default:
		for (t=0;t<10000;t++) zz=alea(0,1,param.randCase); 
		break;
	}

	//-------------------------------------------- Define the problem
	functionCode =func[indFunc];

	pb=problemDef(functionCode);  

	// Automatically estimate the population size
	param.S=sqrt(40*40 + (pb.SS.D+2)*(pb.SS.D+2)); 
	z=(40 + 2 * sqrt(pb.SS.D)); // B
	param.S=max(param.S,z); 

	if(param.S>S_max) ERROR("Swarm size too big. Increase S_max");
	if(pb.SS.D>DMax) ERROR ("Can't solve it. You should increase DMax");	

	// Some information
	printf ("\n Function %i ", functionCode);

	// ----------------------------------------------- RUNS	 
	errorMean = 0;	    
	evalMean = 0;	    
	nFailure = 0;	

	for (run = 0; run < runMax; run++)  
	{	
	 // Initialise positions
	 R.SW.S=param.S;
	 if(pb.SS.normalise>0) {xMin=0; xMax=pb.SS.normalise;} // [0,normalise]^D

	 for (s = 0; s < R.SW.S; s++)   
	 {
		R.SW.X[s].size=pb.SS.D;
		R.SW.X[s].out=0; // Just for compatibility with the Research version
		
		for (d = 0; d < pb.SS.D; d++)  
		{  
		 if(pb.SS.normalise==0) { xMin=pb.SS.min[d]; xMax=pb.SS.max[d];}
		 R.SW.X[s].x[d] = alea (xMin,xMax,param.randCase);	
		}	
	 }

	 // Take quantisation into account
	 if(pb.SS.quantisation==1)
	 {
		for (s = 0; s < R.SW.S; s++)
		{  
		 R.SW.X[s] = quantis (R.SW.X[s], pb.SS);
		}
	 }
	 R.nEval=R.SW.S;

	 // First evaluations
	 for (s = 0; s < R.SW.S; s++) 
	 {	
		R.SW.X[s].f = perf (R.SW.X[s], pb.function,pb.SS,pb.objective);
		if(R.SW.X[s].f<errMin) errMin=R.SW.X[s].f;
		if(R.SW.X[s].f>errMax) errMax=R.SW.X[s].f;
		//R.SW.P[s] = R.SW.X[s];	// Best position = current one
	 }
	 // --------------------------------------------Optimisation 

	 result = APS1 (param, pb,R, functionCode);
	 error = result.error;

	 if (error > pb.epsilon) // Failure
		nFailure = nFailure + 1;

	 if(pb.SS.normalise>0)
	 {
		for(d=0;d<pb.SS.D;d++)
		 result.SW.X[result.SW.best].x[d]=
		 pb.SS.min[d]+(pb.SS.max[d]-pb.SS.min[d])*result.SW.X[result.SW.best].x[d]
		 /pb.SS.normalise;
	 }		

	 // Memorize the best (useful if more than one run)
	 if(run==0) bestBest=result.SW.X[result.SW.best];
	 else
		if(error<bestBest.f) bestBest=result.SW.X[result.SW.best];

	 // Result display
	 errorMean=errorMean+error;

	 //	if(run==runMax-1) //
	 {
		printf ("\nRun %i. S %i,  Eval %f. Error %e ", run+1, param.S, result.nEval, error);
		printf(" Mean %e",errorMean/(run+1));
		zz=100*(1-(double)nFailure/(run+1));
		printf("  Success  %.2f%%",zz);
		printf(" best particle %i", result.SW.best);

		// Best position display
		//for (d=0;d<pb.SS.D;d++) printf(" %f",result.SW.P[result.SW.best].x[d]);

		// Save result
		fprintf( f_run, "\n%i %.1f %.0f %e %e ", run+1, zz, result.nEval,  error, errorMean/(run+1) );
		// Save best position
		for ( d = 0; d < pb.SS.D; d++ ) fprintf( f_run, " %f",  result.SW.X[result.SW.best].x[d] );
	 }


	 // Compute/save some statistical information
	 if (run == 0)
		errorMin = error;
	 else if (error < errorMin)
		errorMin = error;

	 evalMean = evalMean + result.nEval;	
	 errorMeanBest[run] = error;	
	}		// End loop on "run"

	// ---------------------END 

	// Display some statistical information
	printf("\nFunction %i",functionCode);
	evalMean = evalMean / (double) runMax;   
	errorMean = errorMean / (double) runMax;

	printf ("\n Eval. (mean)= %f", evalMean);	
	printf ("\n Error (mean) = %e", errorMean);

	// Variance
	variance = 0;

	for (run = 0; run < runMax; run++)
	 variance = variance + pow (errorMeanBest[run] - errorMean, 2);

	variance = sqrt (variance / runMax);	    
	printf ("\n Std. dev. %e", variance); 

	// Success rate and minimum value
	printf("\n Failure(s) %i  ",nFailure);
	successRate = 100 * (1 - nFailure / (double) runMax);			
	printf ("Success rate = %.2f%%", successRate);
	success[indFunc]=successRate;

	printf ("\n Best min value = %1.20e", errorMin);
	printf ("\nPosition of the optimum: ");
	for (d=0;d<pb.SS.D;d++) printf(" %.20f",bestBest.x[d]);

	// Save	
	fprintf (f_synth,"%i %i %i %.20f %.20f %3.1f",pb.evalMax,
	         functionCode,  pb.SS.D, errorMean, variance,successRate);
	fprintf(f_synth," %f %.20f",evalMean, bestBest.f);

	for (d=0;d<pb.SS.D;d++) fprintf(f_synth, " %1.20e",bestBest.x[d]);  
	fprintf(f_synth,"\n"); 

	// Specific save for Repulsive problem
	if(pb.function==24)
	{		
	 for (d=0;d<pb.SS.D-1;d=d+2) 
	 {
		fprintf(f_synth, " %1.20e %1.20e",bestBest.x[d],bestBest.x[d+1]);  
		fprintf(f_synth,"\n"); 
	 }	
	}	     	   
 } // End "for ind[..."

 printf("\n errMax : %f",errMax);

 // Repeat informations
 printf("\n---------");
 printf("\n Function(s):");
 for(indFunc=0;indFunc<nbFunc;indFunc++) // Functions codes
 {
	functionCode =func[indFunc];
	printf(" %i",functionCode);
 }
 printf("\n");
 zz=0;
 for(indFunc=0;indFunc<nbFunc;indFunc++) // Success rates
 {
	zz=zz+success[indFunc];
	printf(" %.2f%%",success[indFunc]);
 }

 meanSuccess=zz/nbFunc;
 printf("\n Mean success rate= %f",meanSuccess);
 fprintf(f_synth,". . . %f",meanSuccess);

 printf("\n Population size:  %i", param.S);
 printf("\n tryMore option: %i ", param.tryMore);
 printf("\n %e random numbers have been used",nbRand);
 fprintf(f_run,"\nnbRand %e",nbRand);

 return (0);
}
//==============================================================================
void positionDisplay(struct position X)
{
 int d,D;
 D=X.size;
 printf("\nf= %f x= ",X.f);
 for(d=0;d<D;d++) printf("%f ",X.x[d]);
}
//==============================================================================
void positionSave(struct position X, int i)
{
 int d,D;
 D=X.size;
 fprintf(f_trace,"\n%i: f= %f x= ",i,X.f);
 for(d=0;d<D;d++) fprintf(f_trace,"%f ",X.x[d]);
}
//==============================================================================
#include "alea.c"
#include "perf.c"
#include "problemDef.c"
#include "tools.c"
#include "APS1.c"


