#include "stdio.h"
#include "math.h"
#include <stdlib.h>
#include <time.h>
// For more math, in particular linear algebra
// Do not forget to link to gsl and gslcblas
#include <gsl/gsl_math.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>

#define	DMax 101	// Max number of dimension + 1
#define fMax 6				// Max number of constraints +1
#define funcMax 14		// Max number of functions sequentially solved 
#define R_max 100	// Max number of runs
#define	S_max 150 		// Max swarm size
#define zero  1.0e-40	// To avoid numerical instabilities on some machines
#define infinity 1.e+100
// For the RNG KISS
# define ulong unsigned long // To generate pseudo-random numbers with KISS
# define RAND_MAX_KISS ((unsigned long) 4294967295) //  Needs ISO C90

// For KISS_PRNG
#include <stdint.h>

typedef uint64_t u64;

#define QSIZE 0x200000
#define CNG (cng = 6906969069ULL * cng + 13579)
#define XS (xs ^= (xs << 13), xs ^= (xs >> 17), xs ^= (xs << 43))
#define KISS (B64MWC() + CNG + XS)
long double KISS_PRNG_Max=18446744073709551615.0; //pow(2,64) -1;

// For the RNG Mersenne 64 bits
#define NN 312
#define MM 156
#define MATRIX_A 0xB5026F5AA96619E9ULL
#define UM 0xFFFFFFFF80000000ULL /* Most significant 33 bits */
#define LM 0x7FFFFFFFULL /* Least significant 31 bits */
/* The array for the state vector */
static unsigned long long mt[NN]; 
/* mti==NN+1 means mt[NN] is not initialized */
static int mti=NN+1; 


// For quasi-random numbers generation (option)
#include <gsl/gsl_qrng.h> // Do not forget to link to gsl and gslcblas
#define halton gsl_qrng_halton
#define sobol gsl_qrng_sobol

# define randNbMax 500 // // When using a list of previously generated random numbers
											// Suggestion: 2*DMax * S_max

#define ERROR(a) {printf("\nERROR: %s\n", a);exit(-1);}

// Structures
struct quantum 
{
	double q[DMax];
	int size; 
};

struct SS
{ 
	int D;							// Dimension of the search space (D-rectangle)
	double max[DMax];		// Maximum value on each dimension 
	double min[DMax]; 	// Minimum value on each dimension
	struct quantum q;		// Quantisation step size. 0 => continuous problem
	double normalise; 	//  >0  if normalisation needed on [0,normalise]^D
	int quantisation; 	// Flag. Set to 1 if quantisation needed
};

struct param 
{
 	int randCase;		// Kind of RNG
	int S;					// Population (swarm) size. 
 int tryMore;			// Flag to define which kind of loop is used.
};

struct fitness 
{
	int size; 
	double f[fMax];
};

struct position 
{ 
	double f;				// Fitness value  + constraints <=0
	int size;				// Number of dimensions  D
	double x[DMax];		// Coordinates
 int out;				//Flag. 1 => out of the definition space 
};

struct vector 
{  
	int size;  
	double v[DMax]; 
};

struct vectorList
{
	int size;
	struct vector V[randNbMax];
};

struct problem 
{ 
	double epsilon; 					// Admissible error
	int evalMax; 							// Maximum number of fitness evaluations
	int function; 						// Function code
	double objective; 				// Objective value
	struct SS SS;							// Search space
};

struct swarm 
{ 
	int best; 					// rank of the best particle
	//struct position P[S_max];	// Previous best positions found by each particle
	int S; 						// Swarm size 
	//struct vector V[S_max];	// Velocities
	struct position X[S_max];	// Positions 
};

struct result 
{  
	double nEval; 		// Number of evaluations  
	struct swarm SW;	// Final swarm
	double error;		// Numerical result of the run
};

struct matrix
{
	int D;
	double m[DMax][DMax];
};

struct intDouble
{
	int i;
	double d;
};
// --------------- Sub-programs (lexicographic order)

// In alea.c
double alea (double a, double b, int option);
void aleaIndex(int index[], int S,int option);
int alea_integer (int a, int b,int option);
//double aleaMean(double mean, int option);
double alea_normal (double mean, double stdev,int option);
struct vector alea_sphere(int D, double radius, int option, double mean,double sigma,int randOption);


// In KISS_PRNG.c
u64 B64MWC(void); 
void randk_reset(void);
void randk_seed(void);
void randk_seed_manual(u64 seed);
void randk_warmup(int rounds);
u64 randk(void);


// In APS1.c
struct result APS1(struct param param, struct problem pb, struct result R,
                  int function);

// In lennard_jones.c 
double lennard_jones (struct position x); 

// in main.c
void positionDisplay(struct position X);

// In mersenne.c
void init_genrand64(unsigned long long seed);
void init_by_array64(unsigned long long init_key[],
		     unsigned long long key_length);
unsigned long long genrand64_int64(void);
long long genrand64_int63(void);
double genrand64_real1(void);
double genrand64_real2(void);
double genrand64_real3(void);

// In perf.c
struct fitness constraint(struct position x, int functCode);
double perf (struct position x, int function,struct SS SS, double objective);	

// In problemDef.c
struct problem problemDef(int functionCode);
	// See also cec2005.c, cec2005data.c, cec2005pb.c

// In tools.c
static int compareIntDouble (void const *a, void const *b);
struct position confinement(struct position x0, double xMin[],
                            double xMax[], struct SS SS, int confin);
double distanceL(struct position x1, struct position x2, double L);
struct position eval(struct position x0, 
                          double xMin[], 
                            double xMax[], struct SS SS, int confin, 
                          int function, int quant, double objective);
double fact(int N);
double max(double a, double b);
double min(double a, double b);
struct position quantis (struct position x, struct SS SS ) ;
void resultDisplay(struct result R, int N);
int sign (double x);
double simplexVolum(struct intDouble index[], int D, struct swarm SW);
double deter(struct matrix M); // include


// --------------- Global variables	
long double	E ;  // exp(1). Useful for some test functions
double errMax,errMin;
double FEs;
double nbRand; // For information. Number of random numbers used for
							// sequence of runs
long double	pi ; // Useful for some test functions
double rMax;
double randNumber[randNbMax];
int randRank;

// For KISS PRNG
	static u64 QARY[QSIZE];
static int j;
static u64 carry;
static u64 xs;
static u64 cng;

// For Network problem
int bcsNb;
int btsNb;

// For Repulsion problem
int Dim;

// --------------- Files
FILE * f_proba;
FILE * f_rand;
FILE * f_run;
FILE * f_synth;
FILE * f_trace;
