
//==============================================================================
static int compareIntDouble (void const *a, void const *b)
{
	// For qsort (increasing order of "double")
	struct intDouble const *pa = a;
	struct intDouble const *pb = b;

	if(pa->d > pb->d) return 1;
	if(pa->d < pb->d) return -1;
	return 0;
}
//==============================================================================
struct position confinement(struct position x0, double xMin[], 
                            double xMax[], struct SS SS, int confin)
{
 //Check for constraint violations, and modify if any
 // Note: there are more sophisticated ways
 int d;
 int D;
 struct position x;

 x=x0;
 x.out=0;
 D=x.size;
 switch(confin)
 {
	case 0: // No confinement
	 for (d = 0; d < D; d++)
	{
	 if (x.x[d] < xMin[d])
	 {
		x.out=1;
		return x;}
	 if (x.x[d] > xMax[d]) {
		x.out=1;
		return x;}
	}
	break;

	case 1: // Confinement
	 x.out=0;

	 for (d = 0; d < D; d++)
	{	
	 if (x.x[d] < xMin[d]) x.x[d] = xMin[d];
	 else
	 {
		if (x.x[d] > xMax[d])	x.x[d] = xMax[d];
	 }				
	}	
	 break;
 }
 if(x.out==1) printf("\nWARNING. New position out of the definition space");
 return x;
}
// ===========================================================
double distanceL (struct position x1, struct position x2,double L) 
{  // Distance between two positions
	// L = 2 => Euclidean	
	int d;     
	double n;

	n = 0;

	for (d = 0; d < x1.size; d++)
		n = n + pow (fabs(x1.x[d] - x2.x[d]), L);

	n = pow (n, 1/L);
	return n;    
}
//==============================================================================
struct position eval(struct position x, 
                          double xMin[], 
                            double xMax[], struct SS SS, int confin, 
                          int function, int quant, double objective)
{

		 // Evaluation
		 if(x.out==0)
		 {
			x.f =perf(x,function, SS,objective);	
			FEs = FEs + 1;
		 }
		 else
			x.f=errMax;
 
 return x;
}
//==============================================================================
double fact(int N)
{
	double f;
 int i;
 f=1;
 for(i=1;i<=N;i++) f=f*i;
	return f;
}
// ===========================================================
double max(double a, double b)
{
	if(a>b) return a;
	return b;
}
// ===========================================================
double min(double a, double b)
{
	if(a<b) return a;
	return b;
}
// =============================================================================
struct position quantis (struct position x, struct SS SS ) 
{     
 /*
	Quantisation of a position
	Only values like x+k*q (k integer) are admissible 
	 */ 
 int d;
 double qd;
 struct position quantx;

 quantx = x;     
 for (d = 0; d < x.size; d++)
 {
	qd = SS.q.q[d];	

	if (qd > zero)	// Note that qd can't be < 0
	{           				
	 quantx.x[d] = qd * floor (0.5 + x.x[d] / qd);	    
	}	
 }
 return quantx;    
}
// =============================================================================
void resultDisplay(struct result R, int N)
{
 int d;
 int D;
 int i;
 D=R.SW.X[0].size;
 printf("\n %i points = ",N);
 for(i=0;i<N;i++)
 {
	printf("\n%i, x= ",i);
	for(d=0;d<D;d++) printf("%f ",R.SW.X[i].x[d]);
	printf(" f= %f",R.SW.X[i].f);
 }
}
// ===========================================================
int sign (double x) 
{     
	if (x == 0)	return 0;
	if (x < 0)	return -1;    
	return 1;   
}
//==============================================================================
double simplexVolum(struct intDouble index[], int D, struct swarm SW)
{
		// The simplex is defined by D+1 points, X1, X2, etc.
 int d;
 int j;
 struct matrix M;
 int ri;
 double vol;

		M.D=D+1; // Each column i is Xi plus 1 at the bottom
		for(j=0;j<D+1;j++) // For each column ...
	 {
		for(d=0;d<D;d++) // ... for each line (i.e. each dimension
		{
		 ri=index[j].i;
		 M.m[d][j]=SW.X[ri].x[d];
		}
		// Last line
		M.m[D][j]=1;
	 }
 vol=fabs(deter(M))/fact(D); // volume = abs(determinant(M)). This is a theorem 
 return vol;
}

#include "deter.c"
